import { BaseObject } from './BaseObject';
import { BaseModelJson, IBaseModel, IBaseModelOptions, IConversionOption } from './BaseModel.types';
/**
 * BaseModel is a design pattern used to transfer data between software application subsystems.
 *
 * Note: If the data doesn't match the property names you can set the value manually after update super method has been called.
 * Also in the class you inherit BaseModel from you can override the update method to handle the data how you want.
 *
 * @param [data] {any} Provide a way to update the base model upon initialization.
 * @param [opts] {{ expand:boolean }} Options for the base model.
 * @author Robert S. (www.codeBelt.com)
 * @example
 *      // Example how to extend the BaseModel class.
 *      let data = {
 *              make: 'Tesla',
 *              model: 'Model S',
 *              YeAr: 2014,
 *              feature: {
 *                  abs: true,
 *                  airbags: true
 *              }
 *      }
 *      let carModel = new CarModel(data);
 *
 *
 *      // Example how to extend the BaseModel class.
 *      class CarModel extends BaseModel {
 *
 *          // You need to have properties so the data will get assigned.
 *          // If not the data will not get assigned to the model.
 *          make = null;
 *          model = null;
 *          year = null;
 *          allWheel = false; // Set a default value
 *
 *          // You can assign BaseModel to a property which will
 *          // automatically created it and pass the data to it.
 *          feature = FeatureModel
 *
 *          // If you have an array of data and want them assign to a BaseModel.
 *          feature = [FeatureModel];
 *
 *          constructor(data = {}, opts = {}) {
 *              super(opts);
 *
 *              if (data) {
 *                  this.update(data);
 *              }
 *          }
 *
 *          // @overridden BaseModel.update
 *          update(data) {
 *              super.update(data);
 *
 *              // If the data doesn't match the property name.
 *              // You can set the value(s) manually after the update super method has been called.
 *              this.year = data.YeAr;
 *          }
 *      }
 */
export declare class BaseModel extends BaseObject implements IBaseModel {
    /**
     * This property helps distinguish a BaseModel from other functions.
     */
    static readonly IS_BASE_MODEL: boolean;
    constructor(opts?: IBaseModelOptions);
    /**
     * Provide a way to update the BaseModel.
     *
     * @example
     *     // Example of updating some of the data:
     *     carModel.update({ year: 2015, allWheel: true});
     *
     *     // Of course you can also do it the following way:
     *     carModel.year = 2015;
     *     carModel.allWheel = false;
     */
    update(data?: any, conversionOptions?: IConversionOption): any;
    /**
     * Converts the BaseModel data into a JSON object.
     *
     * @example
     *     const obj = carModel.toJSON();
     */
    toJSON(): BaseModelJson<this>;
    /**
     * Converts a BaseModel to a JSON string,
     *
     *  @example
     *     const str = carModel.toJSONString();
     */
    toJSONString(): string;
    /**
     * Converts the string json data into an Object and calls the {{#crossLink "BaseModel/update:method"}}{{/crossLink}} method with the converted Object.
     *
     * @example
     *      const str = '{"make":"Tesla","model":"Model S","year":2014}'
     *      const carModel = new CarModel();
     *      carModel.fromJSON(str);
     */
    fromJSON(json: string): any;
    /**
     * Create a clone/copy of the BaseModel.
     *
     * @example
     *     const clone = carModel.clone();
     *
     *     const clone = carModel.clone<SomeModel>();
     */
    clone(): this;
    /**
     * Adds the updateData to the property
     */
    protected _getPropertyData(currentPropertyData: any, passedInDataForProperty: any): any;
    protected _updateData(currentPropertyData: any, passedInDataForProperty: any): any;
    /**
     * Check the currentPropertyData to see if it is a BaseModel Class and not instantiated.
     */
    protected _isBaseModelClass(currentPropertyData: any): boolean;
    /**
     * Check the currentPropertyData to see if it is an instantiated version of the BaseModel.
     */
    protected _isBaseModelObject(currentPropertyData: any): boolean;
    /**
     * Check if the data is an object.
     */
    protected _isObject(data: any, consoleError?: boolean): boolean;
}
